@@ -30,9 +30,11 @@ from sas.qtgui.MainWindow.DroppableDataLoadWidget import DroppableDataLoadWidget
 import sas.qtgui.Perspectives as Perspectives
 
 DEFAULT_PERSPECTIVE = "Fitting"
+ANALYSIS_TYPES = ['Fitting (*.fitv)', 'Inversion (*.pr)', 'All File (*.*)']
 
 logger = logging.getLogger(__name__)
 
+
 class DataExplorerWindow(DroppableDataLoadWidget):
     # The controller which is responsible for managing signal slots connections
     # for the gui and providing an interface to the data model.
@@ -108,6 +110,9 @@ class DataExplorerWindow(DroppableDataLoadWidget):
         self.communicator.forcePlotDisplaySignal.connect(self.displayData)
         self.communicator.updateModelFromPerspectiveSignal.connect(self.updateModelFromPerspective)
 
+        # fixing silly naming clash in other managers
+        self.communicate = self.communicator
+
         self.cbgraph.editTextChanged.connect(self.enableGraphCombo)
         self.cbgraph.currentIndexChanged.connect(self.enableGraphCombo)
 
@@ -265,20 +270,25 @@ class DataExplorerWindow(DroppableDataLoadWidget):
         filename = QtWidgets.QFileDialog.getOpenFileName(**kwargs)[0]
         if filename:
             self.default_project_location = os.path.dirname(filename)
+            # Inversion perspective will remove all data with delete
             self.deleteAllItems()
             # Currently project load is available only for fitting
             if self.cbFitting.currentText != DEFAULT_PERSPECTIVE:
-                self.cbFitting.setCurrentIndex(self.cbFitting.findText(DEFAULT_PERSPECTIVE))
+                self.cbFitting.setCurrentIndex(
+                    self.cbFitting.findText(DEFAULT_PERSPECTIVE))
+                # delete all (including the default) tabs
+                self._perspective().deleteAllTabs()
             self.readProject(filename)
 
     def loadAnalysis(self):
         """
         Called when the "Open Analysis" menu item chosen.
         """
+        file_filter = ';;'.join(ANALYSIS_TYPES)
         kwargs = {
             'parent'    : self,
             'caption'   : 'Open Analysis',
-            'filter'    : 'Project (*.fitv);;All files (*.*)',
+            'filter'    : file_filter,
             'options'   : QtWidgets.QFileDialog.DontUseNativeDialog
         }
         filename = QtWidgets.QFileDialog.getOpenFileName(**kwargs)[0]
@@ -308,13 +318,13 @@ class DataExplorerWindow(DroppableDataLoadWidget):
 
         return filename
 
-    def saveAsAnalysisFile(self, tab_id=1):
+    def saveAsAnalysisFile(self, tab_id=1, extension='fitv'):
         """
         Show the save as... dialog and return the chosen filepath
         """
-        default_name = "FitPage"+str(tab_id)+".fitv"
+        default_name = "Analysis"+str(tab_id)+"."+str(extension)
 
-        wildcard = "fitv files (*.fitv)"
+        wildcard = "{0} files (*.{0})".format(extension)
         kwargs = {
             'caption'   : 'Save As',
             'directory' : default_name,
@@ -326,16 +336,16 @@ class DataExplorerWindow(DroppableDataLoadWidget):
         filename = filename_tuple[0]
         return filename
 
-    def saveAnalysis(self, data, tab_id=1):
+    def saveAnalysis(self, data, tab_id=1, ext='fitv'):
         """
         Called when the "Save Analysis" menu item chosen.
         """
-        filename = self.saveAsAnalysisFile(tab_id)
+        filename = self.saveAsAnalysisFile(tab_id, ext)
         if not filename:
             return
         _, extension = os.path.splitext(filename)
         if not extension:
-            filename = '.'.join((filename, 'fitv'))
+            filename = '.'.join((filename, ext))
         self.communicator.statusBarUpdateSignal.emit("Saving analysis... %s\n" % os.path.basename(filename))
 
         with open(filename, 'w') as outfile:
@@ -343,6 +353,35 @@ class DataExplorerWindow(DroppableDataLoadWidget):
 
         self.communicator.statusBarUpdateSignal.emit('Analysis saved.')
 
+    def flatDataForModel(self, model):
+        """
+        Get a flat "name:data1d/2d" dict for all
+        items in the model, including children
+        """
+        all_data = {}
+        for i in range(model.rowCount()):
+            item = model.item(i)
+            data = GuiUtils.dataFromItem(item)
+            if data is None: continue
+            # Now, all plots under this item
+            filename = data.filename
+            all_data[filename] = data
+            other_datas = GuiUtils.plotsFromFilename(filename, model)
+            # skip the main plot
+            other_datas = list(other_datas.values())[1:]
+            for data in other_datas:
+                all_data[data.name] = data
+
+        return all_data
+
+    def getAllFlatData(self):
+        """
+        Get items from both data and theory models
+        """
+        data = self.flatDataForModel(self.model)
+        theory = self.flatDataForModel(self.theory_model)
+        return (data, theory)
+
     def allDataForModel(self, model):
         # data model
         all_data = {}
@@ -400,11 +439,17 @@ class DataExplorerWindow(DroppableDataLoadWidget):
 
     def getAllData(self):
         """
-        converts all datasets into serializable dictionary
+        Get items from both data and theory models
         """
         data = self.allDataForModel(self.model)
         theory = self.allDataForModel(self.theory_model)
+        return (data, theory)
 
+    def getSerializedData(self):
+        """
+        converts all datasets into serializable dictionary
+        """
+        data, theory = self.getAllData()
         all_data = {}
         all_data['is_batch'] = str(self.chkBatch.isChecked())
 
@@ -426,7 +471,7 @@ class DataExplorerWindow(DroppableDataLoadWidget):
 
     def readProject(self, filename):
         """
-        Read out datasets and fitpages from file
+        Read out datasets and perspective information from file
         """
         # Find out the filetype based on extension
         ext = os.path.splitext(filename)[1]
@@ -456,28 +501,38 @@ class DataExplorerWindow(DroppableDataLoadWidget):
                 except Exception as ex:
                     logging.error("Project load failed with " + str(ex))
                     return
+        cs_keys = []
+        visible_perspective = DEFAULT_PERSPECTIVE
         for key, value in all_data.items():
-            if key=='is_batch':
-                self.chkBatch.setChecked(True if value=='True' else False)
-                if 'batch_grid' not in all_data:
-                    continue
-                grid_pages = all_data['batch_grid']
-                for grid_name, grid_page in grid_pages.items():
-                    grid_page.append(grid_name)
-                    self.parent.showBatchOutput(grid_page)
+            if key == 'is_batch':
+                self.chkBatch.setChecked(value == 'True')
+                if 'batch_grid' in all_data:
+                    grid_pages = all_data['batch_grid']
+                    for grid_name, grid_page in grid_pages.items():
+                        grid_page.append(grid_name)
+                        self.parent.showBatchOutput(grid_page)
                 continue
             if 'cs_tab' in key:
+                cs_keys.append(key)
                 continue
+            if 'visible_perspective' in key:
+                visible_perspective = value
             # send newly created items to the perspective
             self.updatePerspectiveWithProperties(key, value)
 
+        # Set to fitting perspective and load in Batch and C&S Pages
+        self.cbFitting.setCurrentIndex(
+            self.cbFitting.findText(DEFAULT_PERSPECTIVE))
         # See if there are any batch pages defined and create them, if so
         self.updateWithBatchPages(all_data)
 
         # Only now can we create/assign C&S pages.
-        for key, value in all_data.items():
-            if 'cs_tab' in key:
-                self.updatePerspectiveWithProperties(key, value)
+        for key in cs_keys:
+            self.updatePerspectiveWithProperties(key, all_data[key])
+
+        # Set to perspective shown when project was saved
+        self.cbFitting.setCurrentIndex(
+                self.cbFitting.findText(visible_perspective))
 
     def updateWithBatchPages(self, all_data):
         """
@@ -519,6 +574,7 @@ class DataExplorerWindow(DroppableDataLoadWidget):
             items = self.updateModelFromData(data_dict)
 
         if 'fit_params' in value:
+            self.cbFitting.setCurrentIndex(self.cbFitting.findText(DEFAULT_PERSPECTIVE))
             params = value['fit_params']
             # Make the perspective read the rest of the read data
             if not isinstance(params, list):
@@ -536,6 +592,11 @@ class DataExplorerWindow(DroppableDataLoadWidget):
                 self.sendItemToPerspective(items[0], tab_index=tab_index)
                 # Assign parameters to the most recent (current) page.
                 self._perspective().updateFromParameters(page)
+        if 'pr_params' in value:
+            self.cbFitting.setCurrentIndex(self.cbFitting.findText('Inversion'))
+            params = value['pr_params']
+            self.sendItemToPerspective(items[0])
+            self._perspective().updateFromParameters(params)
         if 'cs_tab' in key and 'is_constraint' in value:
             # Create a C&S page
             self._perspective().addConstraintTab()
@@ -618,6 +679,9 @@ class DataExplorerWindow(DroppableDataLoadWidget):
                 deleted_names.append(str(self.model.item(ind).text()))
                 deleted_items.append(item)
 
+                # Delete corresponding open plots
+                self.closePlotsForItem(item)
+
                 self.model.removeRow(ind)
                 # Decrement index since we just deleted it
                 ind -= 1
@@ -683,14 +747,13 @@ class DataExplorerWindow(DroppableDataLoadWidget):
 
         if len(selected_items) < 1:
             return
-
-        # Which perspective has been selected?
-        if len(selected_items) > 1 and not self._perspective().allowBatch():
-            if hasattr(self._perspective(), 'title'):
-                title = self._perspective().title()
+        #Check that you have only one box item checked when swaping data
+        if len(selected_items) > 1 and (self.chkSwap.isChecked() or not self._perspective().allowBatch()):
+            if hasattr(self._perspective(), 'name'):
+                title = self._perspective().name
             else:
                 title = self._perspective().windowTitle()
-            msg = title + " does not allow multiple data."
+            msg = title + " does not allow replacing multiple data."
             msgbox = QtWidgets.QMessageBox()
             msgbox.setIcon(QtWidgets.QMessageBox.Critical)
             msgbox.setText(msg)
@@ -700,7 +763,10 @@ class DataExplorerWindow(DroppableDataLoadWidget):
 
         # Notify the GuiManager about the send request
         try:
-            self._perspective().setData(data_item=selected_items, is_batch=self.chkBatch.isChecked())
+            if self.chkSwap.isChecked():
+                self._perspective().swapData(selected_items[0])
+            else:
+                self._perspective().setData(data_item=selected_items, is_batch=self.chkBatch.isChecked())
         except Exception as ex:
             msg = "%s perspective returned the following message: \n%s\n" %(self._perspective().name, str(ex))
             logging.error(msg)
@@ -869,11 +935,13 @@ class DataExplorerWindow(DroppableDataLoadWidget):
         """
         self.manager.add_data(data_list)
 
-    def updateGraphCount(self, graph_list):
+    def updateGraphCount(self, graphs):
         """
         Modify the graph name combo and potentially remove
         deleted graphs
         """
+        graph2, delete = graphs
+        graph_list = PlotHelper.currentPlots()
         self.updateGraphCombo(graph_list)
 
         if not self.active_plots:
@@ -902,6 +970,10 @@ class DataExplorerWindow(DroppableDataLoadWidget):
         """
         self.communicator.perspectiveChangedSignal.emit(self.cbFitting.itemText(index))
         self.chkBatch.setEnabled(self.parent.perspective().allowBatch())
+        # Deactivate and uncheck the swap data option if the current perspective does not allow it
+        self.chkSwap.setEnabled(self.parent.perspective().allowSwap())
+        if not self.parent.perspective().allowSwap():
+            self.chkSwap.setCheckState(False)
 
     def itemFromFilename(self, filename):
         """
@@ -999,7 +1071,10 @@ class DataExplorerWindow(DroppableDataLoadWidget):
         """
         if not hasattr(plot, 'name'):
             return False
-        ids_vals = [val.data[0].name for val in self.active_plots.values()]
+
+        ids_vals = [val.data[0].name for val in self.active_plots.values()
+                    if val.data[0].plot_role != Data1D.ROLE_DATA or
+                    isinstance(val.data[0], Data2D)]
 
         return plot.name in ids_vals
 
@@ -1172,7 +1247,7 @@ class DataExplorerWindow(DroppableDataLoadWidget):
     def readData(self, path):
         """
         verbatim copy-paste from
-           sasgui.guiframe.local_perspectives.data_loader.data_loader.py
+        ``sasgui.guiframe.local_perspectives.data_loader.data_loader.py``
         slightly modified for clarity
         """
         message = ""
@@ -1227,7 +1302,7 @@ class DataExplorerWindow(DroppableDataLoadWidget):
                     if hasattr(item, 'errors'):
                         for error_data in item.errors:
                             data_error = True
-                            message += "\tError: {0}\n".format(error_data)
+                            error_message += "\tError: {0}\n".format(error_data)
                     else:
 
                         logging.error("Loader returned an invalid object:\n %s" % str(item))
@@ -1237,7 +1312,7 @@ class DataExplorerWindow(DroppableDataLoadWidget):
                 logging.error(sys.exc_info()[1])
 
                 any_error = True
-            if any_error or error_message != "":
+            if any_error or data_error or error_message != "":
                 if error_message == "":
                     error = "Error: " + str(sys.exc_info()[1]) + "\n"
                     error += "while loading Data: \n%s\n" % str(basename)
@@ -1473,7 +1548,7 @@ class DataExplorerWindow(DroppableDataLoadWidget):
         self.new_plot = globals()[method_name](self, quickplot=True)
         self.new_plot.data = data
         #new_plot.plot(marker='o')
-        self.new_plot.plot()
+        self.new_plot.plot(data=data)
 
         # Update the global plot counter
         title = "Plot " + data.name
