@@ -44,6 +44,8 @@ from sas.qtgui.Calculators.GenericScatteringCalculator import GenericScatteringC
 from sas.qtgui.Calculators.ResolutionCalculatorPanel import ResolutionCalculatorPanel
 from sas.qtgui.Calculators.DataOperationUtilityPanel import DataOperationUtilityPanel
 
+import sas.qtgui.Plotting.PlotHelper as PlotHelper
+
 # Perspectives
 import sas.qtgui.Perspectives as Perspectives
 from sas.qtgui.Perspectives.Fitting.FittingPerspective import FittingWindow
@@ -51,6 +53,7 @@ from sas.qtgui.MainWindow.DataExplorer import DataExplorerWindow, DEFAULT_PERSPE
 
 from sas.qtgui.Utilities.AddMultEditor import AddMultEditor
 from sas.qtgui.Utilities.ImageViewer import ImageViewer
+from sas.qtgui.Utilities.FileConverter import FileConverterWidget
 
 logger = logging.getLogger(__name__)
 
@@ -116,13 +119,19 @@ class GuiManager(object):
                                               "Tutorial.pdf"))
 
     def info(self, type, value, tb):
-        logger.error("SasView threw exception: " + str(value))
-        traceback.print_exception(type, value, tb)
+        logger.error("".join(traceback.format_exception(type, value, tb)))
 
     def addWidgets(self):
         """
         Populate the main window with widgets
         """
+        # Preload all perspectives
+        loaded_dict = {}
+        for name, perspective in Perspectives.PERSPECTIVES.items():
+            loaded_perspective = perspective(parent=self)
+            loaded_dict[name] = loaded_perspective
+        self.loadedPerspectives = loaded_dict
+
         # Add FileDialog widget as docked
         self.filesWidget = DataExplorerWindow(self._parent, self, manager=self._data_manager)
         ObjectLibrary.addObject('DataExplorer', self.filesWidget)
@@ -176,6 +185,7 @@ class GuiManager(object):
         self.GENSASCalculator = GenericScatteringCalculator(self)
         self.ResolutionCalculator = ResolutionCalculatorPanel(self)
         self.DataOperation = DataOperationUtilityPanel(self)
+        self.FileConverter = FileConverterWidget(self)
 
     def addCategories(self):
         """
@@ -191,6 +201,70 @@ class GuiManager(object):
             logger.error("%s: could not load SasView models")
             logger.error(traceback.format_exc())
 
+    def updatePlotItems(self, graphs):
+        """
+        Wrapper for adding/removing actions in the windows menu
+        """
+        plot, delete = graphs
+        if not plot.data:
+            return
+        if delete:
+            self.removePlotItemsInWindowsMenu(plot)
+        else:
+            self.addPlotItemsInWindowsMenu(plot)
+
+
+    def addPlotItemsInWindowsMenu(self, plot):
+        """
+        Dynamically update the QMenu content and assign signals
+        """
+        if not plot:
+            return
+        name = plot[1].name
+        for action in self._workspace.menuWindow.actions():
+            if action.text() == name:
+                # action with this name already exists
+                return
+
+        # create action for this plot
+        action = self._workspace.menuWindow.addAction(name)
+        # connect action to slot
+        action.triggered.connect(lambda chk, item=name: self.plotSelectedSlot(name))
+        # add action to windows menu
+        self._workspace.menuWindow.addAction(action)
+
+    def plotSelectedSlot(self, plot_name):
+        """
+        Set focus on the selected plot
+        """
+        # loop over all visible plots and find the requested plot
+        for plot in PlotHelper.currentPlots():
+            # take last plot
+            if PlotHelper.plotById(plot).data[-1].name == plot_name:
+                # set focus on the plot
+                # Note: none of the StackOverflow recommended solutions work here!
+                # neither raise_(), nor showNormal() nor setWindowState(Qt.WindowActive)
+                PlotHelper.plotById(plot).showNormal()
+                PlotHelper.plotById(plot).setFocus()
+                return
+        pass
+
+    def removePlotItemsInWindowsMenu(self, plot):
+        """
+        Dynamically update the QMenu content and disconnect signals
+        """
+        if not plot: # watch out for quick plots
+            return
+        name = plot.data[-1].name
+        # loop over actions
+        for action in self._workspace.menuWindow.actions():
+            if action.text() == name:
+                # disconnect action to slot
+                action.triggered.disconnect()
+                self._workspace.menuWindow.removeAction(action)
+                return
+        pass
+
     def updateLogContextMenus(self, visible=False):
         """
         Modify the View/Data Explorer menu item text on widget visibility
@@ -250,18 +324,21 @@ class GuiManager(object):
         """
         Respond to change of the perspective signal
         """
-        # Close the previous perspective
+        # Remove the previous perspective from the window
         self.clearPerspectiveMenubarOptions(self._current_perspective)
         if self._current_perspective:
-            self._current_perspective.setClosable()
+            # Remove perspective and store in Perspective dictionary
+            self.loadedPerspectives[
+                self._current_perspective.name] = self._current_perspective
+            self._workspace.workspace.removeSubWindow(self._current_perspective)
             self._workspace.workspace.removeSubWindow(self.subwindow)
-            self._current_perspective.close()
-        # Default perspective
-        self._current_perspective = Perspectives.PERSPECTIVES[str(perspective_name)](parent=self)
+        # Get new perspective
+        self._current_perspective = self.loadedPerspectives[str(perspective_name)]
 
         self.setupPerspectiveMenubarOptions(self._current_perspective)
 
-        self.subwindow = self._workspace.workspace.addSubWindow(self._current_perspective)
+        self.subwindow = self._workspace.workspace.addSubWindow(
+            self._current_perspective)
 
         # Resize to the workspace height
         workspace_height = self._workspace.workspace.sizeHint().height()
@@ -449,6 +526,8 @@ class GuiManager(object):
         self.communicate.plotRequestedSignal.connect(self.showPlot)
         self.communicate.plotFromFilenameSignal.connect(self.showPlotFromFilename)
         self.communicate.updateModelFromDataOperationPanelSignal.connect(self.updateModelFromDataOperationPanel)
+        self.communicate.activeGraphsSignal.connect(self.updatePlotItems)
+
 
     def addTriggers(self):
         """
@@ -501,6 +580,7 @@ class GuiManager(object):
         self._workspace.actionGeneric_Scattering_Calculator.triggered.connect(self.actionGeneric_Scattering_Calculator)
         self._workspace.actionPython_Shell_Editor.triggered.connect(self.actionPython_Shell_Editor)
         self._workspace.actionImage_Viewer.triggered.connect(self.actionImage_Viewer)
+        self._workspace.actionFile_Converter.triggered.connect(self.actionFile_Converter)
         self._workspace.actionOrientation_Viewer.triggered.connect(self.actionOrientation_Viewer)
         self._workspace.actionFreeze_Theory.triggered.connect(self.actionFreeze_Theory)
         # Fitting
@@ -573,62 +653,50 @@ class GuiManager(object):
         filename = self.filesWidget.saveProject()
 
         # datasets
-        all_data = self.filesWidget.getAllData()
-
-        # fit tabs
-        params={}
-        perspective = self.perspective()
-        if hasattr(perspective, 'isSerializable') and perspective.isSerializable():
-            params = perspective.serializeAllFitpage()
-
-        # project dictionary structure:
-        # analysis[data.id] = [{"fit_data":[data, checkbox, child data],
-        #                       "fit_params":[fitpage_state]}
-        # "fit_params" not present if dataset not sent to fitting
-        analysis = {}
-
+        all_data = self.filesWidget.getSerializedData()
+        final_data = {}
         for id, data in all_data.items():
-            if id=='is_batch':
-                analysis['is_batch'] = data
-                analysis['batch_grid'] = self.grid_window.data_dict
-                continue
-            data_content = {"fit_data":data}
-            if id in params.keys():
-                # this dataset is represented also by the fit tab. Add to it.
-                data_content["fit_params"] = params[id]
-            analysis[id] = data_content
+            final_data[id] = {'fit_data': data}
 
-        # standalone constraint pages
-        for keys, values in params.items():
-            if not 'is_constraint' in values[0]:
-                continue
-            analysis[keys] = values[0]
+        # Save from all serializable perspectives
+        # Analysis should return {data-id: serialized-state}
+        for name, per in self.loadedPerspectives.items():
+            if hasattr(per, 'isSerializable') and per.isSerializable:
+                analysis = per.serializeAll()
+                for key, value in analysis.items():
+                    if key in final_data:
+                        final_data[key].update(value)
+                    elif 'cs_tab' in key:
+                        final_data[key] = value
+
+        final_data['is_batch'] = analysis.get('is_batch', 'False')
+        final_data['batch_grid'] = self.grid_window.data_dict
+        final_data['visible_perspective'] = self._current_perspective.name
 
         with open(filename, 'w') as outfile:
-            GuiUtils.saveData(outfile, analysis)
+            GuiUtils.saveData(outfile, final_data)
 
     def actionSave_Analysis(self):
         """
         Menu File/Save Analysis
         """
         per = self.perspective()
-        if not isinstance(per, FittingWindow):
+        if not hasattr(per, 'isSerializable') or not per.isSerializable:
             return
         # get fit page serialization
-        params = per.serializeCurrentFitpage()
-        # Find dataset ids for the current tab
-        # (can be multiple, if batch)
-        data_id = per.currentTabDataId()
-        tab_id = per.currentTab.tab_id
+        all_data = self.filesWidget.getSerializedData()
         analysis = {}
-        for id in data_id:
-            an = {}
-            data_for_id = self.filesWidget.getDataForID(id)
-            an['fit_data'] = data_for_id
-            an['fit_params'] = [params]
-            analysis[id] = an
-
-        self.filesWidget.saveAnalysis(analysis, tab_id)
+        state = per.serializeCurrentPage()
+        for id, params in state.items():
+            if id in all_data:
+                analysis[id] = {'fit_data': all_data[id]}
+                analysis[id].update(params)
+        if len(analysis) > 0:
+            tab_id = 1 if not hasattr(per,
+                                      'currentTab') else per.currentTab.tab_id
+            self.filesWidget.saveAnalysis(analysis, tab_id, per.ext)
+        else:
+            logger.warning('No analysis was available to be saved.')
 
     def actionQuit(self):
         """
@@ -655,7 +723,7 @@ class GuiManager(object):
         can be saved to the clipboard
         """
         self.communicate.copyFitParamsSignal.emit("")
-        self._workspace.actionPaste.setEnabled(True)
+        #self._workspace.actionPaste.setEnabled(True)
         pass
 
     def actionPaste(self):
@@ -770,7 +838,8 @@ class GuiManager(object):
     def actionData_Operation(self):
         """
         """
-        self.communicate.sendDataToPanelSignal.emit(self._data_manager.get_all_data())
+        data, theory = self.filesWidget.getAllFlatData()
+        self.communicate.sendDataToPanelSignal.emit(dict(data, **theory))
 
         self.DataOperation.show()
 
@@ -854,6 +923,16 @@ class GuiManager(object):
             logging.error(str(ex))
             return
 
+    def actionFile_Converter(self):
+        """
+        Shows the File Converter widget.
+        """
+        try:
+            self.FileConverter.show()
+        except Exception as ex:
+            logging.error(str(ex))
+            return
+
     #============ FITTING =================
     def actionNew_Fit_Page(self):
         """
@@ -1103,6 +1182,8 @@ class GuiManager(object):
         """
         if hasattr(self, "filesWidget"):
             self.filesWidget.displayData(plot, id)
+            # update windows menu
+            self.addPlotItemsInWindowsMenu(plot)
 
     def uncheckAllMenuItems(self, menuObject):
         """
