@@ -14,46 +14,68 @@ from mpl_toolkits.mplot3d import Axes3D
 import path_prepare
 
 # Local
-from sas.qtgui.Plotting.PlotterData import Data1D
+from sas.qtgui.Plotting.PlotterData import Data1D, Data2D
 from sas.sascalc.dataloader.loader import Loader
 from sas.qtgui.MainWindow.DataManager import DataManager
 
 from sas.qtgui.MainWindow.DataExplorer import DataExplorerWindow
 from sas.qtgui.MainWindow.GuiManager import GuiManager
 from sas.qtgui.Utilities.GuiUtils import *
-from UnitTesting.TestUtils import QtSignalSpy
+from sas.qtgui.UnitTesting.TestUtils import QtSignalSpy
 from sas.qtgui.Plotting.Plotter import Plotter
 from sas.qtgui.Plotting.Plotter2D import Plotter2D
 import sas.qtgui.Plotting.PlotHelper as PlotHelper
 
+from sas.sasview import __version__ as SASVIEW_VERSION
+
 if not QApplication.instance():
     app = QApplication(sys.argv)
 
+
+class MyPerspective(object):
+    def __init__(self):
+        self.name = "Dummy Perspective"
+
+    def communicator(self):
+        return Communicate()
+
+    def allowBatch(self):
+        return True
+
+    def allowSwap(self):
+        return True
+
+    def setData(self, data_item=None, is_batch=False):
+        return None
+
+    def swapData(self, data_item=None, is_batch=False):
+        return None
+
+    def title(self):
+        return self.name
+
+
+class dummy_manager(object):
+    def __init__(self):
+        self._perspective = MyPerspective()
+
+    def communicator(self):
+        return Communicate()
+
+    def perspective(self):
+        return self._perspective
+
+    def workspace(self):
+        return None
+
+    class _parent(object):
+        screen_width = 1024
+        screen_height = 768
+
 class DataExplorerTest(unittest.TestCase):
     '''Test the Data Explorer GUI'''
     def setUp(self):
         '''Create the GUI'''
-        class MyPerspective(object):
-            def communicator(self):
-                return Communicate()
-            def allowBatch(self):
-                return True
-            def setData(self, data_item=None, is_batch=False):
-                return None
-            def title(self):
-                return "Dummy Perspective"
-
-        class dummy_manager(object):
-            def communicator(self):
-                return Communicate()
-            def perspective(self):
-                return MyPerspective()
-            def workspace(self):
-                return None
-            class _parent(object):
-                screen_width = 1024
-                screen_height = 768
-
         self.form = DataExplorerWindow(None, dummy_manager())
 
     def tearDown(self):
@@ -77,6 +99,8 @@ class DataExplorerTest(unittest.TestCase):
         self.assertIsInstance(self.form.cmdSendTo.icon(), QIcon)
         self.assertEqual(self.form.chkBatch.text(), "Batch mode")
         self.assertFalse(self.form.chkBatch.isChecked())
+        self.assertEqual(self.form.chkSwap.text(), "Swap data")
+        self.assertFalse(self.form.chkSwap.isChecked())
 
         # Buttons - theory tab
 
@@ -274,12 +298,12 @@ class DataExplorerTest(unittest.TestCase):
         QTest.mouseClick(deleteButton, Qt.LeftButton)
 
 
-    def notestSendToButton(self):
+    def testSendToButton(self):
         """
         Test that clicking the Send To button sends checked data to a perspective
         """
         # Send empty data
-        mocked_perspective = self.form.parent.perspective()
+        mocked_perspective = self.form._perspective()
         mocked_perspective.setData = MagicMock()
 
         # Click on the Send To  button
@@ -287,7 +311,7 @@ class DataExplorerTest(unittest.TestCase):
 
         # The set_data method not called
         self.assertFalse(mocked_perspective.setData.called)
-               
+
         # Populate the model
         filename = ["cyl_400_20.txt"]
         self.form.readData(filename)
@@ -295,8 +319,7 @@ class DataExplorerTest(unittest.TestCase):
         QApplication.processEvents()
 
         # setData is the method we want to see called
-        mocked_perspective = self.form.parent.perspective()
-        mocked_perspective.setData = MagicMock(filename)
+        mocked_perspective.swapData = MagicMock()
 
         # Assure the checkbox is on
         self.form.cbSelect.setCurrentIndex(0)
@@ -306,21 +329,52 @@ class DataExplorerTest(unittest.TestCase):
 
         QApplication.processEvents()
 
-        # Test the set_data method called once
+        # Test the set_data method called
         self.assertTrue(mocked_perspective.setData.called)
+        self.assertFalse(mocked_perspective.swapData.called)
+
+        # Now select the swap data checkbox
+        self.form.chkSwap.setChecked(True)
+
+        # Click on the Send To  button
+        QTest.mouseClick(self.form.cmdSendTo, Qt.LeftButton)
+
+        QApplication.processEvents()
+
+        # Now the swap data method should be called
+        self.assertTrue(mocked_perspective.setData.called_once)
+        self.assertTrue(mocked_perspective.swapData.called)
+
+        # Test the exception block
+        QMessageBox.exec_ = MagicMock()
+        QMessageBox.setText = MagicMock()
+        mocked_perspective.swapData = MagicMock(side_effect = Exception("foo"))
+
+        # Click on the button to so the mocked swapData method raises an exception
+        QTest.mouseClick(self.form.cmdSendTo, Qt.LeftButton)
+
+        # Assure the message box popped up
+        QMessageBox.exec_.assert_called_once()
+        # With the right message
+        QMessageBox.setText.assert_called_with("foo")
 
         # open another file
         filename = ["cyl_400_20.txt"]
         self.form.readData(filename)
 
-        # Mock the warning message
-        QMessageBox = MagicMock()
+        # Mock the warning message and the swapData method
+        QMessageBox.exec_ = MagicMock()
+        QMessageBox.setText = MagicMock()
+        mocked_perspective.swapData = MagicMock()
 
-        # Click on the button
+        # Click on the button to swap both datasets to the perspective
         QTest.mouseClick(self.form.cmdSendTo, Qt.LeftButton)
 
         # Assure the message box popped up
-        QMessageBox.assert_called_once()
+        QMessageBox.exec_.assert_called_once()
+        # With the right message
+        QMessageBox.setText.assert_called_with(
+            "Dummy Perspective does not allow replacing multiple data.")
 
     def testDataSelection(self):
         """
@@ -532,6 +586,9 @@ class DataExplorerTest(unittest.TestCase):
         p_file="cyl_400_20.txt"
         output_object = loader.load(p_file)
         new_data = [(None, manager.create_gui_data(output_object[0], p_file))]
+        _, test_data = new_data[0]
+        self.assertTrue(f'Data file generated by SasView v{SASVIEW_VERSION}' in
+                        test_data.notes)
 
         # Mask retrieval of the data
         test_patch.return_value = new_data
@@ -917,6 +974,99 @@ class DataExplorerTest(unittest.TestCase):
         self.assertEqual(len(PlotHelper.currentPlots()), 0)
         self.assertEqual(len(self.form.plot_widgets), 0)
 
+    def testPlotsFromMultipleData1D(self):
+        """
+        Tests interplay between plotting 1D datasets and plotting
+        a single 1D dataset from two separate fit tabs
+        GH issue 1546
+        """
+        # prepare active_plots
+        plot1 = Plotter(parent=self.form)
+        data1 = Data1D()
+        data1.name = 'p1'
+        data1.plot_role = Data1D.ROLE_DATA
+        plot1.data = data1
+
+        plot2 = Plotter(parent=self.form)
+        data2 = Data1D()
+        data2.name = 'M2 [p1]'
+        data2.plot_role = Data1D.ROLE_DEFAULT
+        plot2.data = data2
+
+        plot3 = Plotter(parent=self.form)
+        data3 = Data1D()
+        data3.name = 'Residuals for M2[p1]'
+        data3.plot_role = Data1D.ROLE_RESIDUAL
+        plot3.data = data3
+
+        # pretend we're displaying three plots
+        self.form.active_plots['p1'] = plot1
+        self.form.active_plots['M2 [p1]'] = plot2
+        self.form.active_plots['Residuals for M2[p1]'] = plot3
+
+        # redoing plots from the same tab
+        # data -> must be shown
+        self.assertFalse(self.form.isPlotShown(data1))
+
+        # model and residuals are already shown
+        self.assertTrue(self.form.isPlotShown(data2))
+        self.assertTrue(self.form.isPlotShown(data3))
+
+        # Try from different fit page
+        plot4 = Plotter(parent=self.form)
+        data4 = Data1D()
+        data4.name = 'M1 [p1]'
+        data4.plot_role = Data1D.ROLE_DEFAULT
+        plot4.data = data1
+        # same data but must show, since different model
+        self.assertFalse(self.form.isPlotShown(data4))
+
+    def testPlotsFromMultipleData2D(self):
+        """
+        Tests interplay between plotting 2D datasets and plotting
+        a single 2D dataset from two separate fit tabs
+        GH issue 1546
+        """
+        # prepare active_plots
+        plot1 = Plotter(parent=self.form)
+        data1 = Data2D()
+        data1.name = 'p1'
+        data1.plot_role = Data1D.ROLE_DATA
+        plot1.data = data1
+
+        plot2 = Plotter(parent=self.form)
+        data2 = Data2D()
+        data2.name = 'M2 [p1]'
+        data2.plot_role = Data1D.ROLE_DEFAULT
+        plot2.data = data2
+
+        plot3 = Plotter(parent=self.form)
+        data3 = Data2D()
+        data3.name = 'Residuals for M2[p1]'
+        data3.plot_role = Data1D.ROLE_RESIDUAL
+        plot3.data = data3
+
+        # pretend we're displaying three plots
+        self.form.active_plots['p1'] = plot1
+        self.form.active_plots['M2 [p1]'] = plot2
+        self.form.active_plots['Residuals for M2[p1]'] = plot3
+
+        # redoing plots from the same tab
+        # data -> Already there, don't show
+        self.assertTrue(self.form.isPlotShown(data1))
+
+        # model and residuals are already shown
+        self.assertTrue(self.form.isPlotShown(data2))
+        self.assertTrue(self.form.isPlotShown(data3))
+
+        # Try from different fit page
+        plot4 = Plotter(parent=self.form)
+        data4 = Data2D()
+        data4.name = 'M1 [p1]'
+        plot4.data = data1
+        # same data but must show, since different model
+        self.assertFalse(self.form.isPlotShown(data4))
+
 
 if __name__ == "__main__":
     unittest.main()
