@@ -25,6 +25,8 @@ def _legendResize(width, parent):
     resize factor for the legend, based on total canvas width
     """
     # The factor 4.0 was chosen to look similar in size/ratio to what we had in 4.x
+    if not hasattr(parent.parent, "manager"):
+        return None
     if parent is None or parent.parent is None or parent.parent.manager is None \
         or parent.parent.manager.parent is None or parent.parent.manager.parent._parent is None:
         return None
@@ -70,7 +72,6 @@ class PlotterWidget(PlotterBase):
 
         parent.geometry()
 
-
     @property
     def data(self):
         return self._data
@@ -227,7 +228,6 @@ class PlotterWidget(PlotterBase):
         self.plot_lines[data.name] = line
 
         # Now add the legend with some customizations.
-
         if self.showLegend:
             width=_legendResize(self.canvas.size().width(), self.parent)
             if width is not None:
@@ -271,14 +271,20 @@ class PlotterWidget(PlotterBase):
         self.actionAddText = self.contextMenu.addAction("Add Text")
         self.actionRemoveText = self.contextMenu.addAction("Remove Text")
         self.contextMenu.addSeparator()
+        if self.show_legend:
+            self.actionToggleLegend = self.contextMenu.addAction("Toggle Legend")
+            self.contextMenu.addSeparator()
         self.actionChangeScale = self.contextMenu.addAction("Change Scale")
         self.contextMenu.addSeparator()
         self.actionSetGraphRange = self.contextMenu.addAction("Set Graph Range")
         self.actionResetGraphRange =\
             self.contextMenu.addAction("Reset Graph Range")
+
         # Add the title change for dialogs
         self.contextMenu.addSeparator()
         self.actionWindowTitle = self.contextMenu.addAction("Window Title")
+        self.contextMenu.addSeparator()
+        self.actionToggleMenu = self.contextMenu.addAction("Toggle Navigation Menu")
 
         # Define the callbacks
         self.actionAddText.triggered.connect(self.onAddText)
@@ -287,6 +293,9 @@ class PlotterWidget(PlotterBase):
         self.actionSetGraphRange.triggered.connect(self.onSetGraphRange)
         self.actionResetGraphRange.triggered.connect(self.onResetGraphRange)
         self.actionWindowTitle.triggered.connect(self.onWindowsTitle)
+        self.actionToggleMenu.triggered.connect(self.onToggleMenu)
+        if self.show_legend:
+            self.actionToggleLegend.triggered.connect(self.onToggleLegend)
 
     def addPlotsToContextMenu(self):
         """
@@ -354,6 +363,10 @@ class PlotterWidget(PlotterBase):
         self.actionToggleGrid.triggered.connect(self.onGridToggle)
         self.actionChangeScale.triggered.connect(self.onScaleChange)
 
+        if self.show_legend:
+            self.actionToggleLegend = self.contextMenu.addAction("Toggle Legend")
+            self.actionToggleLegend.triggered.connect(self.onToggleLegend)
+
     def onScaleChange(self):
         """
         Show a dialog allowing axes rescaling
@@ -638,6 +651,17 @@ class PlotterWidget(PlotterBase):
         # Plot the line
         self.plot(data=self.fit_result, marker='-', hide_error=True)
 
+    def onToggleLegend(self):
+        """
+        Toggle legend visibility in the chart
+        """
+        if not self.showLegend:
+            return
+
+        visible = self.legend.get_visible()
+        self.legend.set_visible(not visible)
+        self.canvas.draw_idle()
+
     def onMplMouseDown(self, event):
         """
         Left button down and ready to drag
