@@ -7,27 +7,13 @@ Created on Nov 29, 2016
 @author: wpotrzebowski
 '''
 
-import json
-import platform
-import logging
 import os
 import sys
-
+import warnings
 import wx
-
-try:
-    import pyopencl as cl
-except ImportError:
-    cl = None
-
 import sasmodels
-import sasmodels.model_test
-import sasmodels.sasview_model
-
 from sas.sasgui.guiframe.documentation_window import DocumentationWindow
 
-logger = logging.getLogger(__name__)
-
 
 
 class CustomMessageBox(wx.Dialog):
@@ -101,7 +87,7 @@ class GpuOptions(wx.Dialog):
         boxsizer = wx.BoxSizer(orient=wx.VERTICAL)
         self.option_button = {}
         self.buttons = []
-        #Check if SAS_OPENCL is already set as environment variable
+        #Check if SAS_OPENCL is already set as enviromental variable
         self.sas_opencl = os.environ.get("SAS_OPENCL", "")
 
         for clopt in clinfo:
@@ -150,9 +136,6 @@ class GpuOptions(wx.Dialog):
         self.Bind(wx.EVT_BUTTON, self.on_help, help_btn)
 
         test_text = wx.StaticText(self, -1, "WARNING: Running tests can take a few minutes!")
-        test_text2 = wx.StaticText(self, -1, "NOTE: No test will run if No OpenCL is checked")
-        test_text.SetForegroundColour(wx.RED)
-        self.vbox.Add(test_text2, 0, wx.LEFT|wx.EXPAND|wx.ADJUST_MINSIZE, 10)
         self.vbox.Add(test_text, 0, wx.LEFT|wx.EXPAND|wx.ADJUST_MINSIZE, 10)
 
         btn_sizer = wx.BoxSizer(wx.HORIZONTAL)
@@ -180,19 +163,11 @@ class GpuOptions(wx.Dialog):
         """
         clinfo = []
         platforms = []
-
-        if cl is None:
-            logger.warn("Unable to import the pyopencl package.  It may not "
-                        "have been installed.  If you wish to use OpenCL, try "
-                        "running pip install --user pyopencl")
-        else:
-            try:
-                platforms = cl.get_platforms()
-            except cl.LogicError as err:
-                logger.warn("Unable to fetch the OpenCL platforms.  This likely "
-                            "means that the opencl drivers for your system are "
-                            "not installed.")
-                logger.warn(err)
+        try:
+            import pyopencl as cl
+            platforms = cl.get_platforms()
+        except ImportError:
+            warnings.warn("pyopencl import failed. Using only CPU computations")
 
         p_index = 0
         for platform in platforms:
@@ -238,7 +213,12 @@ class GpuOptions(wx.Dialog):
         else:
             if "SAS_OPENCL" in os.environ:
                 del os.environ["SAS_OPENCL"]
-        sasmodels.sasview_model.reset_environment()
+
+        #Sasmodels kernelcl doesn't exist when initiated with None
+        if 'sasmodels.kernelcl' in sys.modules:
+            sasmodels.kernelcl.ENV = None
+
+        reload(sasmodels.core)
         event.Skip()
 
     def on_reset(self, event):
@@ -254,6 +234,10 @@ class GpuOptions(wx.Dialog):
         """
         Run sasmodels check from here and report results from
         """
+        import json
+        import platform
+        #import sasmodels
+
         #The same block of code as for OK but it is needed if we want to have
         #active response to Test button
         no_opencl_msg = False
@@ -264,22 +248,33 @@ class GpuOptions(wx.Dialog):
         else:
             if "SAS_OPENCL" in os.environ:
                 del os.environ["SAS_OPENCL"]
-        sasmodels.sasview_model.reset_environment()
+
+        #Sasmodels kernelcl doesn't exist when initiated with None
+        if 'sasmodels.kernelcl' in sys.modules:
+            sasmodels.kernelcl.ENV = None
+
+
+        #Need to reload sasmodels.core module to account SAS_OPENCL = "None"
+        reload(sasmodels.core)
+
+
+        from sasmodels.model_test import model_tests
 
         try:
-            env = sasmodels.kernelcl.environment()
+            from sasmodels.kernelcl import environment
+            env = environment()
             clinfo = [(ctx.devices[0].platform.vendor,
                        ctx.devices[0].platform.version,
                        ctx.devices[0].vendor,
                        ctx.devices[0].name,
                        ctx.devices[0].version)
                       for ctx in env.context]
-        except Exception:
+        except ImportError:
             clinfo = None
 
         failures = []
         tests_completed = 0
-        for test in sasmodels.model_test.model_tests():
+        for test in model_tests():
             try:
                 test()
             except Exception:
