@@ -5,7 +5,7 @@ import wx
 import sys
 
 from sas.sasgui.guiframe.utils import format_number
-from invariant_widgets import OutputTextCtrl
+from .invariant_widgets import OutputTextCtrl
 # Dimensions related to chart
 RECTANGLE_WIDTH = 400.0
 RECTANGLE_HEIGHT = 20
@@ -421,7 +421,7 @@ class InvariantDetailsPanel(wx.Dialog):
             scale = float(percentage)
         except:
             scale = RECTANGLE_SCALE
-            self.warning_msg += "Receive an invalid scale for %s\n"
+            self.warning_msg += "Recieve an invalid scale for %s\n"
             self.warning_msg += "check this value : %s\n" % str(percentage)
         return  scale
 
