@@ -226,7 +226,7 @@ class Graph(object):
         min_value = None
         max_value = None
         for p in self.plottables:
-            if p.hidden:
+            if p.hidden == True:
                 continue
             if p.x is not None:
                 for x_i in p.x:
@@ -241,7 +241,7 @@ class Graph(object):
         # If the user has set a custom color, ensure the new plot is the same color
         selected_color = plottable.custom_color
         selected_plottable = None
-        for p in self.plottables.keys():
+        for p in list(self.plottables.keys()):
             if plottable.id == p.id:
                 selected_plottable = p
                 if selected_color is None:
@@ -388,7 +388,7 @@ class Transform(object):
         user, along with an indication of which plottable was at fault.
 
         """
-        raise NotImplemented, "Not a valid transform"
+        raise NotImplemented("Not a valid transform")
 
     # Related issues
     # ==============
@@ -516,7 +516,7 @@ class Plottable(object):
             if n == 1:
                 label_dict[collection[0]] = basename
             else:
-                for i in xrange(len(collection)):
+                for i in range(len(collection)):
                     label_dict[collection[i]] = "%s %d" % (basename, i)
         return label_dict
 
@@ -688,17 +688,17 @@ class View(object):
             if dx is not None and not len(dx) == 0 and not len(x) == len(dx):
                 msg = "Plottable.View: Given x and dx are not"
                 msg += " of the same length"
-                raise ValueError, msg
+                raise ValueError(msg)
             # Check length of y array
             if not len(y) == len(x):
                 msg = "Plottable.View: Given y "
                 msg += "and x are not of the same length"
-                raise ValueError, msg
+                raise ValueError(msg)
 
             if dy is not None and not len(dy) == 0 and not len(y) == len(dy):
                 msg = "Plottable.View: Given y and dy are not of the same "
                 msg += "length: len(y)=%s, len(dy)=%s" % (len(y), len(dy))
-                raise ValueError, msg
+                raise ValueError(msg)
             self.x = []
             self.y = []
             if has_err_x:
@@ -733,15 +733,15 @@ class View(object):
             if not len(self.x) == len(self.y):
                 msg = "Plottable.View: transformed x "
                 msg += "and y are not of the same length"
-                raise ValueError, msg
+                raise ValueError(msg)
             if has_err_x and not (len(self.x) == len(self.dx)):
                 msg = "Plottable.View: transformed x and dx"
                 msg += " are not of the same length"
-                raise ValueError, msg
+                raise ValueError(msg)
             if has_err_y and not (len(self.y) == len(self.dy)):
                 msg = "Plottable.View: transformed y"
                 msg += " and dy are not of the same length"
-                raise ValueError, msg
+                raise ValueError(msg)
             # Check that negative values are not plot on x and y axis for
             # log10 transformation
             self.check_data_logX()
@@ -813,7 +813,7 @@ class View(object):
                         tempdy.append(self.dy[i])
                 except:
                     logger.error("check_data_logX: skipping point x %g", self.x[i])
-                    logger.error(sys.exc_value)
+                    logger.error(sys.exc_info()[1])
             self.x = tempx
             self.y = tempy
             self.dx = tempdx
@@ -843,7 +843,7 @@ class View(object):
                         tempdy.append(self.dy[i])
                 except:
                     logger.error("check_data_logY: skipping point %g", self.y[i])
-                    logger.error(sys.exc_value)
+                    logger.error(sys.exc_info()[1])
 
             self.x = tempx
             self.y = tempy
@@ -1061,7 +1061,7 @@ class Data1D(Plottable):
         """
         Renders the plottable on the graph
         """
-        if self.interactive:
+        if self.interactive == True:
             kw['symbol'] = self.symbol
             kw['id'] = self.id
             kw['hide_error'] = self.hide_error
@@ -1107,7 +1107,7 @@ class Theory1D(Plottable):
         """
         Plottable.__init__(self)
         msg = "Theory1D is no longer supported, please use Data1D and change symbol.\n"
-        raise DeprecationWarning, msg
+        raise DeprecationWarning(msg)
 
 class Fit1D(Plottable):
     """
